/******************************************************************************
 * @file additional_sf_ops.h
 *
 * This header file contains includes and function prototypes of the functions
 * defined in the 'additional_sf_ops.c'.
 *
 * <pre>
 * MODIFICATION HISTORY:
 *
 * Ver   Who  Date     Changes
 * ----- ---- -------- -----------------------------------------------
 * 1.00a nagy 05/21/11 First release
 *
 * </pre>
 *
 *****************************************************************************/

#ifndef ADDITIONAL_SF_OPS_H	/* prevent circular inclusions */
#define ADDITIONAL_SF_OPS_H	/* by using protection macros */

#ifdef __cplusplus
extern "C" {
#endif

/***************************** Include Files *********************************/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "xquad_spi.h"
#include <xilmfs.h>

/************************** Constant Definitions *****************************/

// the start address where indexing data is stored (XML) on Serial Flash
#define XML_START_ADDR		0xFF0000

// Serial Flash memory sector size
#define FLASH_SECTOR_SIZE 	0x10000

// Serial Flash memory size minus the area where indexing data (XML) is stored
#define FLASH_SIZE			0x1000000-FLASH_SECTOR_SIZE

/************************** Function Prototypes ******************************/

int WriteXmlLastEntry(unsigned long QSpiBaseAddr, const char *xmlFilename,
		const char *filename, const char *location, const char *size);
int DeleteXmlLastEntry(unsigned long QSpiBaseAddr, const char *xmlFilename,
		const char *filename);

int ReadFileFromSf(unsigned long QSpiBaseAddr, char *buf, const char *location,
		const char *size);
int WriteFileToSf(unsigned long QSpiBaseAddr, char *buf, const char *location,
		int size);
int DeleteFileFromSf(unsigned long QSpiBaseAddr, const char *location,
		const char *size);

int CopyXmlFromSf(unsigned long QSpiBaseAddr, const char *xmlFilename);
int WriteXmlToSf(unsigned long QSpiBaseAddr, const char *xmlFilename, int decSize);

long htoi(char s[]);
long hchartoi(char hexdig, int pos);


#ifdef __cplusplus
}
#endif

#endif /* end of protection macro */
