/*
 * Copyright (c) 2009 Xilinx, Inc.  All rights reserved.
 *
 * Xilinx, Inc.
 * XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" AS A
 * COURTESY TO YOU.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
 * ONE POSSIBLE   IMPLEMENTATION OF THIS FEATURE, APPLICATION OR
 * STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS IMPLEMENTATION
 * IS FREE FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE RESPONSIBLE
 * FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE FOR YOUR IMPLEMENTATION.
 * XILINX EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH RESPECT TO
 * THE ADEQUACY OF THE IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO
 * ANY WARRANTIES OR REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
 * FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include <string.h>
#include "mfs_config.h"
#include "lwip/inet.h"
#include "webserver.h"
#include "additional_sf_ops.h"
#include "lwip/tcp.h"
#include "platform_gpio.h"

// Serial Flash memory controller's base address
#define QSPI_BASE	XPAR_D_QSPI_AXI_0_BASEADDR

char *notfound_header =
	"<html> \
	<head> \
		<title>404</title> \
  		<style type=\"text/css\"> \
		div#request {background: #eeeeee} \
		</style> \
	</head> \
	<body> \
	<h1>404 Page Not Found</h1> \
	<div id=\"request\">";

char *notfound_footer =
	"</div> \
	</body> \
	</html>";

extern int btn_prss, sw_prss;
extern int verbose;

int addrCntWr = 0;
int addrCntRd = 0;

int buc = 0;
int iStart = 0;
int iBuf = 0;

char *sizeToCallback;
char *whereTo;

err_t data_receive_callback(void *arg, struct tcp_pcb * tpcb, struct pbuf * p, err_t err);


// dynamically generate 404 response: this inserts the original
// request string in betwween the notfound_header & footer strings
int do_404(struct tcp_pcb *pcb, char *req, int rlen)
{
	int len, hlen;
	int BUFSIZE = 1024;
	char buf[BUFSIZE];
	err_t err;

	len = strlen(notfound_header) + strlen(notfound_footer) + rlen;

	hlen = generate_http_header(buf, "html", len, 0);

	if (tcp_sndbuf(pcb) < hlen)
	{
		xil_printf("cannot send 404 message, tcp_sndbuf = %d bytes, message "
				"length = %d bytes\r\n", tcp_sndbuf(pcb), hlen);
		return -1;
	}
	if ((err = tcp_write(pcb, buf, hlen, 1)) != ERR_OK)
	{
		xil_printf("%s: error (%d) writing 404 http header\r\n", __FUNCTION__, err);
		return -1;
	}
	tcp_write(pcb, notfound_header, strlen(notfound_header), 1);
	tcp_write(pcb, req, rlen, 1);
	tcp_write(pcb, notfound_footer, strlen(notfound_footer), 1);

	return 0;
}

int do_xml_data(struct tcp_pcb * pcb)
{
	int hlen;
	char * buf, * hbuf;

	// Allocate memory
	buf = (char *)malloc(400);
	hbuf = (char *)malloc(100);
	memset(buf, 0, 400);
	memset(hbuf, 0, 100);

	strcat(buf, "<DATA>\r\n");
	strcat(buf, "    <SET>\r\n");
	sprintf(buf+19, "        <PUSHBUTTONS_STATE>%03d</PUSHBUTTONS_STATE>\r\n", btn_prss);
	sprintf(buf+71, "        <SWITCHES_STATE>%03d</SWITCHES_STATE>\r\n", sw_prss);
	strcat(buf, "    </SET>\r\n");
	strcat(buf, "</DATA>");

	hlen = generate_http_header(hbuf, "xml", strlen(buf), 0);
	tcp_write(pcb, hbuf, hlen, 1);
	tcp_write(pcb, buf, strlen(buf), 1);

	free(hbuf);
	free(buf);

	return strlen(buf);
}

// XXX: do_http_post
int do_http_post(struct tcp_pcb *pcb, char *req)
{
	int BUFSIZE = 1024;
	char *buf;
	char *size, *location, *filename;
	int len, fnReturn;

	buf = (char *)malloc(BUFSIZE);
	memset(buf, 0, BUFSIZE);

    // Check if a "Led" request arrived
    fnReturn = is_cmd_led(req);

    // allocate memory for name and size
    filename = (char *)malloc(24);
    memset(filename, 0, 24);
    location = (char *)malloc(7);
    memset(location, 0, 7);
    size = (char *)malloc(7);
    memset(size, 0, 7);

    if(fnReturn)
    {
    	switch(fnReturn)
    	{
    		case 0xE0: toggle_leds(0, 0);
    			break;
    		case 0xF0: toggle_leds(0, 1);
    		    break;
    		case 0xE1: toggle_leds(1, 0);
    			break;
    		case 0xF1: toggle_leds(1, 1);
    		    break;
    		case 0xE2: toggle_leds(2, 0);
    			break;
    		case 0xF2: toggle_leds(2, 1);
    		    break;
    		case 0xE3: toggle_leds(3, 0);
    			break;
    		case 0xF3: toggle_leds(3, 1);
    		    break;
    		case 0xE4: toggle_leds(4, 0);
    			break;
    		case 0xF4: toggle_leds(4, 1);
    		    break;
    		case 0xE5: toggle_leds(5, 0);
    			break;
    		case 0xF5: toggle_leds(5, 1);
    		    break;
    		case 0xE6: toggle_leds(6, 0);
    			break;
    		case 0xF6: toggle_leds(6, 1);
    		    break;
    		case 0xE7: toggle_leds(7, 0);
    			break;
    		case 0xF7: toggle_leds(7, 1);
    			break;
    	}
    	len = generate_http_header(buf, "js", 1, 0);
    	len++;

    	if(verbose) xil_printf("\nhttp POST: ledstatus: %x", fnReturn);

    	if(tcp_write(pcb, buf, len, 1) != ERR_OK)
    	{
    		xil_printf("error writing http POST response to socket\n");
    		xil_printf("http header = %s\r\n", buf);
    		free(buf);
    		free(filename);
    		free(location);
    		free(size);
    		return -1;
    	}
    }
    else if(is_cmd_upload(req, filename, location, size))
    {
    	int ret;

    	xil_printf("\nUploading...");

    	ret = CopyXmlFromSf(QSPI_BASE, "upload.xml");
    	if(ret == -1)
    	{
    		// no XML file on Flash so create one
    		if(!WriteXmlLastEntry(QSPI_BASE, "upload.xml",
    				filename, "0000000", size))
    		{
    			free(buf);
    			free(filename);
    			free(location);
    			free(size);
    			return -1;
    		}

    		// write XML file back to Flash
    		if(!WriteXmlToSf(QSPI_BASE, "upload.xml", 1))
    		{
    			free(buf);
    			free(filename);
    			free(location);
    			free(size);
    		    return -1;
    		}
    	}
    	else if(ret == 0)
    	{
    		free(buf);
    		free(filename);
    		free(location);
    		free(size);
    		return -1;
    	}
    	else // XML present on Flash
    	{
    		// add the new entry
    		if(!WriteXmlLastEntry(QSPI_BASE, "upload.xml",
    				filename, location, size))
    		{
    			free(buf);
    			free(filename);
    			free(location);
    			free(size);
    			return -1;
    		}

    		// write XML file back to Flash
    		if(!WriteXmlToSf(QSPI_BASE, "upload.xml", 1))
    		{
    			free(buf);
    			free(filename);
    			free(location);
    			free(size);
    			return -1;
    		}
    	}

    	sizeToCallback = size;
    	whereTo = location;

    	// reset byte counter
    	addrCntWr = 0;

    	// receive data now
    	tcp_recv(pcb, data_receive_callback);
    }
    else
    {
    	xil_printf("\nhttp POST: unsupported command");
    }

	// free-up allocated memory
	free(buf);
	free(filename);
	free(location);
	free(size);

	return 0;
}

// callback function to be called when data is received
err_t data_receive_callback(void *arg, struct tcp_pcb *tpcb, struct pbuf *p, err_t err)
{
	struct tcp_pcb *pcb;
	char *xbuf, *buf;
	char *startPtr, *endPtr;
	int fsize, jmpOffset;

	// get the file size
	fsize = htoi(sizeToCallback);

	// allocate memory for the buffer to write to Flash
	xbuf = (char *)malloc(257);
	memset(xbuf, 0, sizeof(xbuf));

	// first write
	if(iBuf == 0)
	{
		// calculate header size
		memcpy(xbuf, ((char *)p->payload), 256);
		startPtr = xbuf;
		endPtr = strstr(xbuf, "Cache-Control: no-cache");
		jmpOffset = endPtr - startPtr;
		jmpOffset += 27;
		memset(xbuf, 0, 257);
		iStart = jmpOffset;
	}
	else
	{
		iStart = 0;
	}
	iBuf++;

	// chop every packet into small 256 bytes batches to write to Flash
	while((iStart + 256)<(p->tot_len))
	{
		memcpy(xbuf, ((char *)p->payload) + iStart, 256);
		if(!WriteFileToSf(QSPI_BASE, xbuf, whereTo, 256))
		{
			free(xbuf);
			return -1;
		}
		memset(xbuf, 0, 257);
		iStart += 256;
		buc += 256;
	}
	// and write the remaining bytes too
	if((p->tot_len) - iStart)
	{
		memcpy(xbuf, ((char *)p->payload) + iStart, (p->tot_len) - iStart);
		if(!WriteFileToSf(QSPI_BASE, xbuf, whereTo, (p->tot_len) - iStart))
		{
			free(xbuf);
			return -1;
		}
		memset(xbuf, 0, 257);
		buc += (p->tot_len) - iStart;
	}

	// free temporary buffer
	free(xbuf);

	// end of stream
	if(buc >= fsize)
	{
		buf = (char *)malloc(400);
		memset(buf, 0, 400);
		// acknowledge received bytes
		tcp_recved(tpcb, p->len);
		//pbuf_free(p);
		int len = generate_http_header(buf, "htm", fsize, 0);
		if((err = tcp_write(pcb, buf, len, 1)) != ERR_OK)
		{
//			xil_printf("\nerror (%d) writing http header to socket", err);
//			xil_printf("\nattempted to write #bytes = %d, tcp_sndbuf = %d",
//						len, tcp_sndbuf(pcb));
			free(buf);
			return -1;
		}
		free(buf);
		// close connection
		//tcp_close(tpcb);
		addrCntWr =0;
		buc = 0;
		iBuf = 0;
		iStart = 0;
		xil_printf("\nUpload complete!");
	}

	// acknowledge received bytes
	tcp_recved(tpcb, p->len);

	// free chain buffer
	pbuf_free(p);

	return ERR_OK;
}

// respond to a GET file download request
// XXX: do_download_file
int do_download_file(struct tcp_pcb *pcb, char *req)
{
	int i=0, n=0, hlen;
	char *startPtr, *filename, *location, *size;
	char *fext, *buf;
	err_t err;

	// a 'download' command should look like this:
	// GET /download/Name=abc.jjj&Location=xxxxxxx&Size=xxxxxxx

	// go past the 'GET /' command
	req += 5;
	if(strncmp(req, "download", 8))
	{
		// not mostly an error (could also
		// be another type of GET request)
		return 0;
	}
	req += 9;

	if(!strncmp(req, "Name=", 5))
	{
		// store the name
		req += 5;
		startPtr = req;
		while(strncmp(req+i, "&Location", 9))
		{
			if(i>23)
			{
				xil_printf("\nThe name of the "
						"file in request is too long");
				return -1;
				break;
			}
			i++;
		}
		req = startPtr;
		filename = (char *)malloc(i);
		memset(filename, 0, i);
		strncpy(filename, req, i);
		filename[i] = 0;
	}
	else
	{
		xil_printf("\nCannot find 'Name' in request");
		return -1;
	}

	// move pointer to the 'location'
	// and store the location of the file
	req += i + 10;
	location = (char *)malloc(7);
	memset(location, 0, 7);
	strncpy((char *)location, req, 7);
	location[7] = 0;
	req += 7;
	if(strncmp(req, "&Size", 5))
	{
		xil_printf("\nCannot find 'Size' in request");
		return -1;
	}

	// move pointer to 'size' now
	req += 6;
	size = (char *)malloc(7);
	memset(size, 0, 7);
	strncpy((char *)size, req, 7);
	size[7] = 0;

	// get a pointer to file extension
	fext = get_file_extension(filename);

	// write the http headers
	int intSize = (int)htoi(size);

	xil_printf("\niSize = %d", intSize);

	hlen = generate_http_header(buf, fext, intSize, 0);
	if((err = tcp_write(pcb, buf, hlen, 1)) != ERR_OK)
	{
		xil_printf("error (%d) writing http header to socket\n", err);
		xil_printf("attempted to write #bytes = %d, tcp_sndbuf = %d\n",
				hlen, tcp_sndbuf(pcb));
		xil_printf("http header = %s\n", buf);
		return -1;
	}

	// allocate 256 bytes for reading
	buf = (char *)malloc(257);
	memset(buf, 0, 257);

	// reset byte counter
	addrCntRd = 0;

	n = intSize;

	// now write the file
	while(n)
	{
		int sndbuf;
		sndbuf = tcp_sndbuf(pcb);

		if(sndbuf<256)
		{
			// not enough space in sndbuf, so send remaining bytes when
			// there is space this is done by storing the fd in as part
			// of the tcp_arg, so that the sent callback handler knows
			// to send data
			http_arg *a = (http_arg *)pcb->callback_arg;
			a->fd = 1;
			a->fsize = intSize;
			//err = tcp_output(pcb);
			xil_printf("\nsndbuf = %d, err = %d", sndbuf, err);

			//return;
		}
		xil_printf("\nsndbuf = %d, n = %d", sndbuf, n);

		/*if(sndbuf > 256)*/ n = ReadFileFromSf(QSPI_BASE, buf, location, size);

		if((err = tcp_write(pcb, buf, n, 1)) != ERR_OK)
		{
			xil_printf("\nerror writing file (%s) to socket, remaining "
					"unwritten bytes = %d", filename, size-n);
			xil_printf("\nattempted to lwip_write %d bytes, tcp write "
					"error = %d", n, err);
			break;
		}
		//tcp_output(pcb);
	}

	// free the name and data buffers
	free(filename);
	free(location);
	free(size);
	free(buf);

	xil_printf("\nDONE SENDING FILE");
	return 1;
}

// respond to a GET 'delete' request
// XXX: do_delete_file
int do_delete_file(struct tcp_pcb *pcb, char *req, int rlen)
{
	char *buf, *startPtr, *endPtr;
	char *filename, *location, *size;
	int hlen, fd, i;
	err_t err;

	// since 'delete' file is only supported for the last
	// written file, a 'delete' command should look like this:
	// GET /delete/Name=xxxxx.aaa&Location=xxxxxxx&Size=xxxxxxx

	// go past the 'GET /' command
	req += 5;
	if(strncmp(req, "delete", 6))
	{
		// not mostly an error (could also
		// be another type of GET request)
		return 0;
	}
	req += 7;

	if(!strncmp(req, "Name=", 5))
	{
		// jump over 'Name='
		req += 5;

		// get the name's length
		i = 0;
		startPtr = req;
		endPtr = strstr(req, "&Location");
		if(endPtr == NULL)
		{
			xil_printf("\nFile name is too long in request");
			return -1;
		}
		i = endPtr - startPtr;

		// now store the file name
		filename = (char *)malloc(i);
		memset(filename, 0, i);
		strncpy(filename, startPtr, i);
		filename[i] = 0;

		// reset pointer to the end of file name
		req = endPtr;
	}
	else
	{
		xil_printf("\nCannot find 'Name' in request");
		return -1;
	}

	if(!strncmp(req, "&Location=", 10))
	{
		// jump over '&Location='
		req += 10;

		// store the location
		location = (char *)malloc(7);
		memset(location, 0, 7);
		strncpy(location, req, 7);
		location[7] = 0;

		// jump over location
		req += 7;
	}
	else
	{
		xil_printf("\nCannot find 'Location' in request");
		free(filename);
		return -1;
	}

	if(!strncmp(req, "&Size", 5))
	{
		// jump over '&Size'
		req += 5;

		// and store file size
		size = (char *)malloc(7);
		memset(size, 0, 7);
		strncpy(size, req, 7);
		size[7] = 0;
	}
	else
	{
		xil_printf("\nCannot find 'Size' in request");
		free(filename);
		free(location);
		return -1;
	}

	xil_printf("\nDeleting...");

	// delete the entry too
	if(!DeleteXmlLastEntry(QSPI_BASE, "delete.xml", filename))
	{
		free(filename);
		free(location);
		free(size);
		return -1;
	}

	buf = (char *)malloc(100);
	memset(buf, 0, 100);

	// write the http headers
	hlen = generate_http_header(buf, "htm", 10, 0);
	if((err = tcp_write(pcb, buf, hlen, 1)) != ERR_OK)
	{
		xil_printf("\nerror (%d) writing http header to socket", err);
		xil_printf("\nattempted to write #bytes = %d, tcp_sndbuf = %d",
				hlen, tcp_sndbuf(pcb));
		free(filename);
		free(location);
		free(size);
		free(buf);
		return -1;
	}

	// delete the file from Flash
	if(!DeleteFileFromSf(QSPI_BASE, location, size))
	{
		free(filename);
		free(location);
		free(size);
		return -1;
	}

	// free up allocated space for buffers
	free(filename);
	free(location);
	free(size);
	free(buf);

	xil_printf("\nDelete complete!");

	return 1;
}

// respond to a GET 'file list' request
//XXX: do_list_file
int do_list_file(struct tcp_pcb *pcb, char *req, int rlen)
{
	int fd, hlen, fsize;
	int ret;
	char *buf;
	err_t err;

	// a 'file list' command should look like this:
	// GET /flash/file/list

	// go past the 'GET /' command
	req += 5;
	if(strncmp(req, "flash/file/list", 15))
	{
		// not mostly an error (could also
		// be another type of GET request)
		return 1;
	}

	// copy XML from Flash
	ret = CopyXmlFromSf(QSPI_BASE, "list_file.xml");
	if(!ret || (ret == -1))
	{
		// XML not present in Flash
		do_404(pcb, req, rlen);
		return -1;
	}

	//open XML file and determine its size
	fd = mfs_file_open("list_file.xml", MFS_MODE_READ);
	fsize = mfs_file_lseek(fd, 0, MFS_SEEK_END);

	// allocate memory for the read buffer
	buf = (char *)malloc(1401);
	memset(buf, 0, 1401);

	// write the http headers
	hlen = generate_http_header(buf, "xml", fsize, 0);
	if ((err = tcp_write(pcb, buf, hlen, 1)) != ERR_OK)
	{
		xil_printf("error (%d) writing http header to socket\r\n", err);
		xil_printf("attempted to write #bytes = %d, tcp_sndbuf = %d\r\n",
				hlen, tcp_sndbuf(pcb));
		xil_printf("http header = %s\r\n", buf);
		free(buf);
		return -1;
	}

	while(fsize)
	{
		int sndbuf, n;
		sndbuf = tcp_sndbuf(pcb);

		if(sndbuf < 1400)
		{
			// not enough space in sndbuf, so send remaining bytes when
			// there is space this is done by storing the fd in as part
			// of the tcp_arg, so that the sent callback handler knows to
			// send data
			http_arg *a = (http_arg *)pcb->callback_arg;
			a->fd = fd;
			a->fsize = fsize;
			return;
		}

		n = mfs_file_read(fd, buf, fsize);

		if((err = tcp_write(pcb, buf, n, 1)) != ERR_OK)
		{
			xil_printf("error writing file (%s) to socket, remaining "
					"unwritten bytes = %d\r\n", "data.xml", fsize - n);
			xil_printf("attempted to lwip_write %d bytes, tcp write "
					"error = %d\r\n", n, err);
			break;
		}
		fsize -= n;
	}

	// close the file
	mfs_file_close(fd);

	// free up read buffer
	free(buf);

	// delete the file from MFS
	mfs_delete_file("list_file.xml");

	if(verbose) xil_printf("\nDone sending XML file");

	return 0;
}

// respond for a file GET request
int do_http_get(struct tcp_pcb *pcb, char *req, int rlen)
{
	int BUFSIZE = 4000;
	char filename[MAX_FILENAME];
	char *buf;
	int fsize, hlen, n;
	int fd, ret;
	char *fext;
	err_t err;

	// call download file if a 'download' command arrived
	ret = do_download_file(pcb, req);//, rlen);
	if(ret == -1)
	{
		// error
		return -1;
	}
	else if(ret)
	{
		return 0;
	}

	// call delete file if a 'delete' command arrived
	ret = do_delete_file(pcb, req, rlen);
	if(ret == -1)
	{
		// error
		return -1;
	}
	else if(ret)
	{
		return 0;
	}

	// call list file if a 'flash/file/list' command arrived
	ret = do_list_file(pcb, req, rlen);
	if(ret == -1)
	{
		// error
		return -1;
	}
	else if(!ret)
	{
		return 0;
	}

	// determine file name
	extract_file_name(filename, req, rlen, MAX_FILENAME);

	// read the state of the switches/pushbuttons
	if(!strncmp(filename, "data.xml", 8))
	{
		do_xml_data(pcb);
		return 0;
	}

	// respond with 404 if not present
	if (mfs_exists_file(filename) != 1)
	{
		xil_printf("\nRequested file %s not found, returning 404", filename);
		do_404(pcb, req, rlen);
		return -1;
	}

	// debug statement on UART
	if(verbose) xil_printf("\nhttp GET: %s", filename);

	// get a pointer to file extension
	fext = get_file_extension(filename);

	// open the file
	fd = mfs_file_open(filename, MFS_MODE_READ);

	// obtain file size, note that lseek with offset 0,
	// MFS_SEEK_END does not move file pointer
	fsize = mfs_file_lseek(fd, 0, MFS_SEEK_END);

	// now allocate memory for the read buffer
	buf = (char *)malloc(BUFSIZE);
	memset(buf, 0, BUFSIZE);

	// write the http headers
	hlen = generate_http_header(buf, fext, fsize, 0);
	if((err = tcp_write(pcb, buf, hlen, 1)) != ERR_OK)
	{
		xil_printf("error (%d) writing http header to socket\r\n", err);
		xil_printf("attempted to write #bytes = %d, tcp_sndbuf = %d\r\n",
				hlen, tcp_sndbuf(pcb));
		xil_printf("http header = %s\r\n", buf);
		free(buf);
		return -1;
	}

	// now write the file
	while (fsize)
	{
		int sndbuf;
		sndbuf = tcp_sndbuf(pcb);

		if (sndbuf < BUFSIZE)
		{
			// not enough space in sndbuf, so send remaining bytes when
			// there is space this is done by storing the fd in as part
			// of the tcp_arg, so that the sent callback handler knows to
			// send data
			http_arg *a = (http_arg *)pcb->callback_arg;
			a->fd = fd;
			a->fsize = fsize;
			return;
		}

		n = mfs_file_read(fd, (char *)buf, BUFSIZE);

		if ((err = tcp_write(pcb, buf, n, 1)) != ERR_OK)
		{
			xil_printf("error writing file (%s) to socket, remaining "
					"unwritten bytes = %d\r\n", filename, fsize - n);
			xil_printf("attempted to lwip_write %d bytes, tcp write "
					"error = %d\r\n", n, err);
			break;
		}
		fsize -= n;
	}

	// free buffer
	free(buf);

	// and close the file
	mfs_file_close(fd);

	return 0;
}

enum http_req_type { HTTP_GET, HTTP_POST, HTTP_UNKNOWN };
enum http_req_type decode_http_request(char *req, int l)
{
	char *get_str = "GET";
	char *post_str = "POST";

	if (!strncmp(req, get_str, strlen(get_str)))
		return HTTP_GET;

	if (!strncmp(req, post_str, strlen(post_str)))
		return HTTP_POST;

	return HTTP_UNKNOWN;
}

// generate and write out an appropriate response for the http request
// this assumes that tcp_sndbuf is high enough to send at least 1 packet
int generate_response(struct tcp_pcb *pcb, char *http_req, int http_req_len)
{
	enum http_req_type request_type = decode_http_request(http_req, http_req_len);

	switch(request_type)
	{
		case HTTP_GET:
			return do_http_get(pcb, http_req, http_req_len);
		case HTTP_POST:
			return do_http_post(pcb, http_req);
		default:
			//xil_printf("request_type != GET|POST\n");
			return do_404(pcb, http_req, http_req_len);
	}
}
