/*
 * Copyright (c) 2009 Xilinx, Inc.  All rights reserved.
 *
 * Xilinx, Inc.
 * XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" AS A
 * COURTESY TO YOU.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
 * ONE POSSIBLE   IMPLEMENTATION OF THIS FEATURE, APPLICATION OR
 * STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS IMPLEMENTATION
 * IS FREE FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE RESPONSIBLE
 * FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE FOR YOUR IMPLEMENTATION.
 * XILINX EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH RESPECT TO
 * THE ADEQUACY OF THE IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO
 * ANY WARRANTIES OR REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
 * FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "platform_gpio.h"
#include "xparameters.h"
#include "xio.h"

#if defined(XPAR_LEDS_8BITS_BASEADDR)
#define LED_BASE XPAR_LEDS_8BITS_BASEADDR
#elif defined(XPAR_LEDS_4BITS_BASEADDR)
#define LED_BASE XPAR_LEDS_4BITS_BASEADDR
#elif defined(XPAR_LEDS_6BIT_BASEADDR)
#define LED_BASE XPAR_LEDS_6BIT_BASEADDR
#else
#define NO_GPIOS
#endif

#if defined(XPAR_DIP_SWITCHES_8BITS_BASEADDR)
#define DIP_BASE XPAR_DIP_SWITCHES_8BITS_BASEADDR
#elif defined(XPAR_DIP_SWITCHES_4BITS_BASEADDR)
#define DIP_BASE XPAR_DIP_SWITCHES_4BITS_BASEADDR
#elif defined(XPAR_PUSH_BUTTONS_POSITION_BASEADDR)
#define DIP_BASE XPAR_PUSH_BUTTONS_POSITION_BASEADDR 
#elif defined(XPAR_DIP_SWITCHES_4BIT_BASEADDR)
#define DIP_BASE XPAR_DIP_SWITCHES_4BIT_BASEADDR
#else
#define NO_GPIOS
#endif

#if defined(XPAR_PUSH_BUTTONS_5BITS_BASEADDR)
#define BTN_BASE XPAR_PUSH_BUTTONS_5BITS_BASEADDR
#endif

volatile unsigned int currState = 0;

void
platform_init_gpios()
{
#ifndef NO_GPIOS
    /* set led gpio data direction to output */
    *(volatile unsigned int *)(LED_BASE + 4) = 0;

    /* set pushbutton gpio data direction to in */
    *(volatile unsigned int *)(BTN_BASE + 4) = ~0;

    /* set dip switch gpio data direction to in */
    *(volatile unsigned int *)(DIP_BASE + 4) = ~0;

    /* initialize leds to OFF */
    *(volatile int *)(LED_BASE) = 0;
#endif
}

void toggle_leds(int LedNr, int State)
{
#ifndef NO_GPIOS
	currState = State ? currState | (1 << LedNr):currState & ~(1 << LedNr);
	*(volatile int *)(LED_BASE) = currState;
#endif
}

unsigned int get_switch_state()
{
#ifdef NO_GPIOS
    return 0x00;
#else
    return *(volatile unsigned int *)(DIP_BASE);
#endif
}

unsigned int get_pushbutton_state()
{
	volatile unsigned int btnTemp, btnMask;

#ifdef NO_GPIOS
    return 0x00;
#else
    /*
     * The following code does a bit swap of bit 2 and bit 0 to account for a
     * mixup in how the buttons are attached to the axi_gpio core.
     */
    btnTemp = *(volatile unsigned int *)(BTN_BASE);
    btnMask = btnTemp & 0b11010;
    if (btnTemp & 0b00100)
    {
    	btnMask |= 0b00001;
    }
    if (btnTemp & 0b00001)
	{
		btnMask |= 0b00100;
	}
    return btnMask;
#endif
}
