/*
 * Copyright (c) 2009 Xilinx, Inc.  All rights reserved.
 *
 * Xilinx, Inc.
 * XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" AS A
 * COURTESY TO YOU.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
 * ONE POSSIBLE   IMPLEMENTATION OF THIS FEATURE, APPLICATION OR
 * STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS IMPLEMENTATION
 * IS FREE FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE RESPONSIBLE
 * FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE FOR YOUR IMPLEMENTATION.
 * XILINX EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH RESPECT TO
 * THE ADEQUACY OF THE IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO
 * ANY WARRANTIES OR REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
 * FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include <string.h>

#include "lwip/sys.h"

#include "mfs_config.h"
#include "webserver.h"
#include "platform_fs.h"
#include "platform_gpio.h"

extern int verbose;

/* Function to check if the request is a "Led" one
 * and if it is, then check the LED's state.
 *
 * return values: 	00 = not a "Led" request
 *					F0 = LED 0, on
 *					E0 = LED 0, off
 *					F1 = LED 1, on
 *					E1 = LED 1, off
 *					F2 = LED 2, on
 *					E2 = LED 2, off
 *					...............
 *					F7 = LED 7, on
 *					E7 = LED 7, off
 */
int is_cmd_led(char * buf)
{
	// skip past "POST /"
	buf += 6;

	if(!strncmp(buf, "Led", 3)) // check for Led/LedNr=x&State=y
	{
		if(!strncmp(buf + 18, "true", 4))
		{
			if(!strncmp(buf + 10, "0", 1))
				return 0xF0;
			else if(!strncmp(buf + 10, "1", 1))
				return 0xF1;
			else if(!strncmp(buf + 10, "2", 1))
				return 0xF2;
			else if(!strncmp(buf + 10, "3", 1))
				return 0xF3;
			else if(!strncmp(buf + 10, "4", 1))
				return 0xF4;
			else if(!strncmp(buf + 10, "5", 1))
				return 0xF5;
			else if(!strncmp(buf + 10, "6", 1))
				return 0xF6;
			else if(!strncmp(buf + 10, "7", 1))
				return 0xF7;
			else
				return 0x00;
		}
		else if(!strncmp(buf + 18, "false", 5))
		{
			if(!strncmp(buf + 10, "0", 1))
				return 0xE0;
			else if(!strncmp(buf + 10, "1", 1))
				return 0xE1;
			else if(!strncmp(buf + 10, "2", 1))
				return 0xE2;
			else if(!strncmp(buf + 10, "3", 1))
				return 0xE3;
			else if(!strncmp(buf + 10, "4", 1))
				return 0xE4;
			else if(!strncmp(buf + 10, "5", 1))
				return 0xE5;
			else if(!strncmp(buf + 10, "6", 1))
				return 0xE6;
			else if(!strncmp(buf + 10, "7", 1))
				return 0xE7;
			else
				return 0x00;
		}
		else
			return 0x00;
	}
	else
		return 0x00;
}

/* Function to check if the request is a "upload" one.
 * The upload request should look like this:
 * POST /upload/Name=adc.jjj&Location=xxxxxxx&Size=xxxxxxx
 *
 * return values: 	0 - not an upload request, or error
 * 					1 - an upload request
 */
int is_cmd_upload(char *req, char *filename, char *location, char *size)
{
	unsigned int i=0;
	char *start, *end;

	// skip past "POST /"
	req += 6;
	if(strncmp(req, "upload", 6))
	{
		return 0;
	}
	req += 7;
	if(!strncmp((const char *)req, "Name=", 5))
	{
		// store the name
		req += 5;
		i = 0;

		// set pointer to the start of the name and search for the
		// '&Size' string, after the end of the name
		start = req;
		end = strstr(req, "&Location");
		if(end == NULL)
		{
			xil_printf("\nSize of the name in request is too long");
			return 0;
		}

		// get the length of 'name'
		i = end-start;

		if(verbose)
		{
			xil_printf("\n#################################");
			xil_printf("\nName length is: %d", i);
		}

		// copy the name to 'filename'
		strncpy(filename, start, i);
		filename[i] = 0;
	}
	else
	{
		xil_printf("\nCannot find 'Name' in request");
		return 0;
	}

	// jump pointer after the '&Location=' string
	req = end;
	req += 10;
	i = 0;

	start = req;
	end = strstr(req, "&Size");
	if(end == NULL)
	{
		xil_printf("\nLocation of the file in request is too long");
		return 0;
	}
	i = end-start;
	if(verbose) xil_printf("\nLocation length is: %d", i);

	// store location
	strcpy(location, "0000000");
	strncpy(location+7-i, start, i);

	// jump pointer after the '&Size=' string
	req = end;
	req += 6;
	i = 0;

	// set pointer to the start of the size and search for
	// 'HTTP' string coming after size and space character
	start = req;
	end = strstr(req, "HTTP");
	end -= 1;
	if(end == NULL)
	{
		xil_printf("\nSize of the file in request is too long");
		return 0;
	}

	// get the length of 'size'
	i = end-start;
	if(verbose) xil_printf("\nSize length is: %d", i);

	// fill with '0' if size length is less than 7
	strcpy(size, "0000000");
	strncpy(size+7-i, start, i);

	if(verbose)
	{
		xil_printf("\nName: |%s|\nLocation: |%s|\nSize: |%s|",
				filename, location, size);
		xil_printf("\n#################################\n\n");
	}

	return 1;
}

void extract_file_name(char *filename, char *req, int rlen, int maxlen)
{
	char *fstart, *fend;

	/* first locate the file name in the request */
	/* requests are of the form GET /path/to/filename HTTP... */

	req += strlen("GET ");

	if (*req == '/')
		req++;

	fstart = req;	/* start marker */

	while (*req != ' ')	/* file name finally ends in a space */
		req++;

	fend = req-1;	/* end marker */

	if (fend < fstart) {
		strcpy(filename, "index.html");
		return;
	}

	/* make sure filename is of reasonable size */
	if (fend - fstart > maxlen) {
		*fend = 0;
		strcpy(filename, "404.html");
		printf("Request filename is too long, length = %d, file = %s "
				"(truncated), max = %d\r\n", (fend - fstart), fstart, maxlen);
		return;
	}

	/* copy over the filename */
	strncpy(filename, fstart, fend-fstart+1);
	filename[fend-fstart+1] = 0;

	/* if last character is a '/', append index.html */
	if (*fend == '/')
		strcat(filename, "index.html");
}

char *get_file_extension(char *fname)
{
	char *fext = fname + strlen(fname) - 1;

	while (fext > fname) {
		if (*fext == '.')
			return fext + 1;
		fext--;
	}

	return NULL;
}

int generate_http_header(char *buf, char *fext, int fsize, int isUpload)
{
	char lbuf[40];

	strcpy(buf, "HTTP/1.1 200 OK\r\nContent-Type: ");

	if (fext == NULL)
		strcat(buf, "text/html");	/* for unknown types */
	else if (!strncmp(fext, "htm", 3))
		strcat(buf, "text/html");	/* html */
	else if (!strncmp(fext, "jpg", 3))
		strcat(buf, "image/jpeg");
	else if (!strncmp(fext, "gif", 3))
		strcat(buf, "image/gif");
	else if (!strncmp(fext, "json", 4))
		strcat(buf, "application/json");
	else if (!strncmp(fext, "js", 2))
		strcat(buf, "application/javascript");
	else if (!strncmp(fext, "pdf", 2))
		strcat(buf, "application/pdf");
	else if (!strncmp(fext, "xml", 3))
		strcat(buf, "application/xml"); /* xml */
	else if (!strncmp(fext, "css", 2))
		strcat(buf, "text/css");
	else
		strcat(buf, "text/plain");	/* for unknown types */
	strcat(buf, "\r\n");

	sprintf(lbuf, "Content-length: %d", fsize);
	strcat(buf, lbuf);
	strcat(buf, "\r\n");

	if(isUpload)
	{
		strcat(buf, "Connection: Keep-Alive\r\n");
	}
	else
	{
		strcat(buf, "Connection: close\r\n");
	}
	strcat(buf, "\r\n");

	return strlen(buf);
}

static int p_arg_count = 0;

http_arg *palloc_arg()
{
	http_arg *a;
	SYS_ARCH_DECL_PROTECT(lev);
	SYS_ARCH_PROTECT(lev);

	a = malloc(sizeof *a);
	if (!a) {
		print("out of memory, attempted to allocate a http argument structure\r\n");
	} else {
		a->count = p_arg_count++;
		a->fd = -1;
		a->fsize = 0;
	}

	SYS_ARCH_UNPROTECT(lev);

	return a;
}

void pfree_arg(http_arg *arg)
{
	SYS_ARCH_DECL_PROTECT(lev);
	SYS_ARCH_PROTECT(lev);
	free(arg);
	SYS_ARCH_UNPROTECT(lev);
}
